"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getVersionedTestObjectProvider = exports.getDataStoreFactory = exports.TestDataObjectType = void 0;
const common_utils_1 = require("@fluidframework/common-utils");
const test_utils_1 = require("@fluidframework/test-utils");
const test_drivers_1 = require("@fluidframework/test-drivers");
const attributor_1 = require("@fluid-experimental/attributor");
const packageVersion_1 = require("./packageVersion");
const testApi_1 = require("./testApi");
exports.TestDataObjectType = "@fluid-example/test-dataStore";
function createGetDataStoreFactoryFunction(api) {
    class TestDataObject extends api.DataObject {
        get _context() {
            return this.context;
        }
        get _runtime() {
            return this.runtime;
        }
        get _root() {
            return this.root;
        }
    }
    const registryMapping = {};
    for (const value of Object.values(api.dds)) {
        registryMapping[value.getFactory().type] = value.getFactory();
    }
    function convertRegistry(registry = []) {
        const oldRegistry = [];
        for (const [key, factory] of registry) {
            const oldFactory = registryMapping[factory.type];
            if (oldFactory === undefined) {
                throw Error(`Invalid or unimplemented channel factory: ${factory.type}`);
            }
            oldRegistry.push([key, oldFactory]);
        }
        return oldRegistry;
    }
    return function (containerOptions) {
        const registry = convertRegistry(containerOptions === null || containerOptions === void 0 ? void 0 : containerOptions.registry);
        const fluidDataObjectType = containerOptions === null || containerOptions === void 0 ? void 0 : containerOptions.fluidDataObjectType;
        switch (fluidDataObjectType) {
            case undefined:
            case test_utils_1.DataObjectFactoryType.Primed:
                return new api.DataObjectFactory(exports.TestDataObjectType, TestDataObject, [...registry].map((r) => r[1]), {});
            case test_utils_1.DataObjectFactoryType.Test:
                return new api.TestFluidObjectFactory(registry);
            default:
                (0, common_utils_1.unreachableCase)(fluidDataObjectType, `Unknown data store factory type ${fluidDataObjectType}`);
        }
    };
}
// Only support current version, not baseVersion support
exports.getDataStoreFactory = createGetDataStoreFactoryFunction((0, testApi_1.getDataRuntimeApi)(packageVersion_1.pkgVersion));
async function createVersionedFluidTestDriver(baseVersion, driverConfig) {
    var _a;
    const driverApi = (0, testApi_1.getDriverApi)(baseVersion, driverConfig === null || driverConfig === void 0 ? void 0 : driverConfig.version);
    return (0, test_drivers_1.createFluidTestDriver)((_a = driverConfig === null || driverConfig === void 0 ? void 0 : driverConfig.type) !== null && _a !== void 0 ? _a : "local", driverConfig === null || driverConfig === void 0 ? void 0 : driverConfig.config, driverApi);
}
async function getVersionedTestObjectProvider(baseVersion, loaderVersion, driverConfig, runtimeVersion, dataRuntimeVersion) {
    const loaderApi = (0, testApi_1.getLoaderApi)(baseVersion, loaderVersion);
    const containerRuntimeApi = (0, testApi_1.getContainerRuntimeApi)(baseVersion, runtimeVersion);
    const dataRuntimeApi = (0, testApi_1.getDataRuntimeApi)(baseVersion, dataRuntimeVersion);
    const driver = await createVersionedFluidTestDriver(baseVersion, driverConfig);
    const innerRequestHandler = async (request, runtime) => runtime.IFluidHandleContext.resolveHandle(request);
    const getDataStoreFactoryFn = createGetDataStoreFactoryFunction(dataRuntimeApi);
    const containerFactoryFn = (containerOptions) => {
        const dataStoreFactory = getDataStoreFactoryFn(containerOptions);
        const runtimeCtor = (containerOptions === null || containerOptions === void 0 ? void 0 : containerOptions.enableAttribution) === true
            ? (0, attributor_1.mixinAttributor)(containerRuntimeApi.ContainerRuntime)
            : containerRuntimeApi.ContainerRuntime;
        const factoryCtor = (0, test_utils_1.createTestContainerRuntimeFactory)(runtimeCtor);
        return new factoryCtor(exports.TestDataObjectType, dataStoreFactory, containerOptions === null || containerOptions === void 0 ? void 0 : containerOptions.runtimeOptions, [innerRequestHandler]);
    };
    return new test_utils_1.TestObjectProvider(loaderApi.Loader, driver, containerFactoryFn);
}
exports.getVersionedTestObjectProvider = getVersionedTestObjectProvider;
//# sourceMappingURL=compatUtils.js.map